@@ -432,6 +432,10 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         layout.setContentsMargins(self._margin_left, 0, self._margin_right, 0)
         layout.setSpacing(0)
 
+    def closeEvent(self, event):
+        self.on_close()
+        super().closeEvent(event)
+
     # --- Public methods to use
     # ------------------------------------------------------------------------
     def get_plugin(self):
@@ -772,11 +776,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
                 self._update_actions()
 
     def change_visibility(self, enable, force_focus=None):
-        """
-        Dock widget visibility has changed.
-        """
-        is_visible = not self.is_visible
-
+        """Dock widget visibility has changed."""
         if self.dockwidget is None:
             return
 
@@ -789,7 +789,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         raise_and_focus = getattr(self, 'RAISE_AND_FOCUS', None)
 
         if force_focus is None:
-            if raise_and_focus or not is_visible:
+            if raise_and_focus and enable:
                 focus_widget = self.get_focus_widget()
                 if focus_widget:
                     focus_widget.setFocus()
@@ -898,6 +898,13 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
             'A PluginMainWidget subclass must define an `update_actions` '
             f'method! Hint: {type(self)} should implement `update_actions`')
 
+    def on_close(self):
+        """
+        Perform actions before the widget is closed.
+
+        This method **must** only operate on local attributes.
+        """
+        pass
 
 def run_test():
     # Third party imports
