@@ -145,12 +145,15 @@ class SpyderToolbarMixin:
         """
         toolbar.add_item(action_or_widget, section=section, before=before)
 
-    def create_stretcher(self):
+    def create_stretcher(self, id_=None):
         """
         Create a stretcher widget to be used in a Qt toolbar.
         """
         stretcher = QWidget()
         stretcher.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
+
+        if id_ is not None:
+            stretcher.ID = id_
         return stretcher
 
     def create_toolbar(self, name: str) -> QToolBar:
@@ -254,7 +257,7 @@ class SpyderMenuMixin:
         """
         from spyder.api.widgets.menus import SpyderMenu
 
-        menu = SpyderMenu(parent=self, title=text)
+        menu = SpyderMenu(parent=self, title=text, menu_id=name)
         if icon is not None:
             menu.menuAction().setIconVisibleInMenu(True)
             menu.setIcon(icon)
@@ -289,7 +292,7 @@ class SpyderMenuMixin:
         ------
         KeyError
             If either of `name`, `context` or `plugin` keys do not exist in the
-            toolbar registry.
+            menu registry.
         """
         plugin = self.PLUGIN_NAME if plugin is None else plugin
         context = self.CONTEXT_NAME if context is None else context
@@ -350,7 +353,8 @@ class SpyderActionMixin:
                       toggled=None, triggered=None, shortcut_context=None,
                       context=Qt.WidgetWithChildrenShortcut, initial=None,
                       register_shortcut=False, section=None, option=None,
-                      parent=None):
+                      parent=None, register_action=True, overwrite=False,
+                      context_name=None, menurole=None):
         """
         name: str
             unique identifiable name for the action
@@ -389,6 +393,19 @@ class SpyderActionMixin:
             The default value is `False`.
         parent: QWidget (None)
             Define the parent of the widget. Use `self` if not provided.
+        register_action: bool, optional
+            If True, the action will be registered and searchable.
+            The default value is `True`.
+        overwrite: bool, optional
+            If True, in case of action overwriting no warning will be shown.
+            The default value is `False`
+        context_name: Optional[str]
+            Name of the context that holds the action in case of registration.
+            The combination of `name` and `context_name` is unique so trying
+            to register an action with the same `name` and `context_name` will
+            cause a warning unless `overwrite` is set to `True`.
+        menurole: QAction.MenuRole, optional
+            Menu role for the action (it only has effect on macOS).
 
         Notes
         -----
@@ -429,8 +446,11 @@ class SpyderActionMixin:
             option=option,
             id_=name,
             plugin=self.PLUGIN_NAME,
-            context_name=self.CONTEXT_NAME,
-            register_action=True
+            context_name=(
+                self.CONTEXT_NAME if context_name is None else context_name),
+            register_action=register_action,
+            overwrite=overwrite,
+            menurole=menurole
         )
         action.name = name
         if icon_text:
@@ -481,7 +501,7 @@ class SpyderActionMixin:
         ------
         KeyError
             If either of `name`, `context` or `plugin` keys do not exist in the
-            toolbar registry.
+            action registry.
         """
         plugin = self.PLUGIN_NAME if plugin is None else plugin
         context = self.CONTEXT_NAME if context is None else context
@@ -556,14 +576,16 @@ class SpyderWidgetMixin(SpyderActionMixin, SpyderMenuMixin,
     # Context name used to store actions, toolbars, toolbuttons and menus
     CONTEXT_NAME = None
 
-    def __init__(self, class_parent=None):
+    def __init__(self, class_parent=None, configuration=CONF):
         for attr in ['CONF_SECTION', 'PLUGIN_NAME']:
             if getattr(self, attr, None) is None:
                 if hasattr(class_parent, attr):
                     # Inherit class_parent CONF_SECTION/PLUGIN_NAME value
                     setattr(self, attr, getattr(class_parent, attr))
+        if configuration is None:
+            configuration = CONF
 
-        super().__init__()
+        super().__init__(configuration=configuration)
 
     @staticmethod
     def create_icon(name):
