@@ -12,12 +12,13 @@ Configuration manager providing access to user/site/project configuration.
 import logging
 import os
 import os.path as osp
-from typing import Optional, Any
+from typing import Any, Dict, Optional, Set
 import weakref
 
 # Local imports
 from spyder.api.utils import PrefixedTuple
-from spyder.config.base import _, get_conf_paths, get_conf_path, get_home_dir
+from spyder.config.base import (
+    _, get_conf_paths, get_conf_path, get_home_dir, reset_config_files)
 from spyder.config.main import CONF_VERSION, DEFAULTS, NAME_MAP
 from spyder.config.types import ConfigurationKey, ConfigurationObserver
 from spyder.config.user import UserConfig, MultiUserConfig, NoDefault, cp
@@ -39,11 +40,12 @@ class ConfigurationManager(object):
     Configuration manager to provide access to user/site/project config.
     """
 
-    def __init__(self, parent=None, active_project_callback=None):
+    def __init__(self, parent=None, active_project_callback=None,
+                 conf_path=None):
         """
         Configuration manager to provide access to user/site/project config.
         """
-        path = self.get_user_config_path()
+        path = conf_path if conf_path else self.get_user_config_path()
         if not osp.isdir(path):
             os.makedirs(path)
 
@@ -79,6 +81,11 @@ class ConfigurationManager(object):
             remove_obsolete=False,
         )
 
+        # This is useful to know in order to execute certain operations when
+        # bumping CONF_VERSION
+        self.old_spyder_version = (
+            self._user_config._configs_map['spyder']._old_version)
+
         # Store plugin configurations when CONF_FILE = True
         self._plugin_configs = {}
 
@@ -103,6 +110,11 @@ class ConfigurationManager(object):
         # Setup
         self.remove_deprecated_config_locations()
 
+    def unregister_plugin(self, plugin_instance):
+        conf_section = plugin_instance.CONF_SECTION
+        if conf_section in self._plugin_configs:
+            self._plugin_configs.pop(conf_section, None)
+
     def register_plugin(self, plugin_class):
         """Register plugin configuration."""
         conf_section = plugin_class.CONF_SECTION
@@ -132,8 +144,7 @@ class ConfigurationManager(object):
 
             # Recreate external plugin configs to deal with part two
             # (the shortcut conflicts) of spyder-ide/spyder#11132
-            spyder_config = self._user_config._configs_map['spyder']
-            if check_version(spyder_config._old_version, '54.0.0', '<'):
+            if check_version(self.old_spyder_version, '54.0.0', '<'):
                 # Remove all previous .ini files
                 try:
                     plugin_config.cleanup()
@@ -605,8 +616,12 @@ class ConfigurationManager(object):
     def iter_shortcuts(self):
         """Iterate over keyboard shortcuts."""
         for context_name, keystr in self._user_config.items('shortcuts'):
-            context, name = context_name.split('/', 1)
-            yield context, name, keystr
+            if context_name == 'enable':
+                continue
+
+            if 'additional_configuration' not in context_name:
+                context, name = context_name.split('/', 1)
+                yield context, name, keystr
 
         for _, (_, plugin_config) in self._plugin_configs.items():
             items = plugin_config.items('shortcuts')
@@ -623,4 +638,21 @@ class ConfigurationManager(object):
             plugin_config.reset_to_defaults(section='shortcuts')
 
 
-CONF = ConfigurationManager()
+try:
+    CONF = ConfigurationManager()
+except Exception:
+    from qtpy.QtWidgets import QMessageBox
+    from spyder.app.utils import create_application
+    app = create_application()
+    reset_reply = QMessageBox.critical(
+        None, 'Spyder',
+        _("There was an error while loading Spyder configuration options. "
+          "You need to reset them for Spyder to be able to launch.\n\n"
+          "Do you want to proceed?"),
+        QMessageBox.Yes, QMessageBox.No)
+    if reset_reply == QMessageBox.Yes:
+        reset_config_files()
+        QMessageBox.information(
+            None, 'Spyder',
+            _("Spyder configuration files resetted!"))
+    os._exit(0)
