@@ -48,6 +48,7 @@ from spyder.utils.palette import QStylePalette
 from spyder.widgets.browser import FrameWebView
 from spyder.widgets.findreplace import FindReplace
 from spyder.widgets.tabs import Tabs
+from spyder.plugins.ipythonconsole.utils.stdfile import StdFile
 
 
 # Localization
@@ -57,6 +58,17 @@ _ = get_translation('spyder')
 # ---- Constants
 # =============================================================================
 MAIN_BG_COLOR = QStylePalette.COLOR_BACKGROUND_1
+SPYDER_KERNELS_MIN_VERSION = '2.3.0'
+SPYDER_KERNELS_MAX_VERSION = '2.4.0'
+SPYDER_KERNELS_VERSION = (
+    f'>={SPYDER_KERNELS_MIN_VERSION};<{SPYDER_KERNELS_MAX_VERSION}')
+SPYDER_KERNELS_VERSION_MSG = _(
+    '>= {0} and < {1}').format(
+        SPYDER_KERNELS_MIN_VERSION, SPYDER_KERNELS_MAX_VERSION)
+SPYDER_KERNELS_CONDA = (
+    f'conda install spyder-kernels={SPYDER_KERNELS_MIN_VERSION[:-2]}')
+SPYDER_KERNELS_PIP = (
+    f'pip install spyder-kernels=={SPYDER_KERNELS_MIN_VERSION[:-1]}*')
 
 
 class IPythonConsoleWidgetActions:
@@ -151,11 +163,6 @@ class IPythonConsoleWidget(PluginMainWidget):
     This signal will request to change the focus to the plugin.
     """
 
-    sig_editor_focus_requested = Signal()
-    """
-    This signal will request to change the focus to the editor if available.
-    """
-
     sig_edit_goto_requested = Signal((str, int, str), (str, int, str, bool))
     """
     This signal will request to open a file in a given row and column
@@ -174,6 +181,16 @@ class IPythonConsoleWidget(PluginMainWidget):
         requested file.
     """
 
+    sig_edit_new = Signal(str)
+    """
+    This signal will request to create a new file in a code editor.
+
+    Parameters
+    ----------
+    path: str
+        Path to file.
+    """
+
     sig_pdb_state_changed = Signal(bool, dict)
     """
     This signal is emitted when the debugging state changes.
@@ -367,6 +384,12 @@ class IPythonConsoleWidget(PluginMainWidget):
         # See spyder-ide/spyder#11880
         self._init_asyncio_patch()
 
+        self._cached_kernel_properties = None
+
+    def on_close(self):
+        self.mainwindow_close = True
+        self.close_clients()
+
     # ---- PluginMainWidget API and settings handling
     # ------------------------------------------------------------------------
     def get_title(self):
@@ -781,9 +804,10 @@ class IPythonConsoleWidget(PluginMainWidget):
         # Check that we are not triggering validations in the initial
         # notification sent when Spyder is starting or when another option
         # already required a restart and the restart dialog was shown
-        if option in self.initial_conf_options:
-            self.initial_conf_options.remove(option)
-            return
+        if not self._testing:
+            if option in self.initial_conf_options:
+                self.initial_conf_options.remove(option)
+                return
 
         restart_needed = False
         restart_options = []
@@ -807,20 +831,46 @@ class IPythonConsoleWidget(PluginMainWidget):
 
         inline_backend = 0
         pylab_restart = False
-        client_backend_not_inline = [False] * len(self.clients)
+        clients_backend_require_restart = [False] * len(self.clients)
         current_client = self.get_current_client()
-        current_client_backend_not_inline = False
+        current_client_backend_require_restart = False
         if pylab_o and pylab_backend_n == option and current_client:
             pylab_backend_o = self.get_conf(pylab_backend_n)
-            client_backend_not_inline = [
-                client.shellwidget.get_matplotlib_backend() != inline_backend
-                for client in self.clients]
-            current_client_backend_not_inline = (
-                current_client.shellwidget.get_matplotlib_backend()
-                != inline_backend)
-            pylab_restart = (
-                any(client_backend_not_inline) and
-                pylab_backend_o != inline_backend)
+
+            # Check if clients require a restart due to a change in
+            # interactive backend.
+            clients_backend_require_restart = []
+            for client in self.clients:
+                interactive_backend = (
+                    client.shellwidget.get_mpl_interactive_backend())
+
+                if (
+                    # No restart is needed if the new backend is inline
+                    pylab_backend_o != inline_backend and
+                    # There was an error getting the interactive backend in
+                    # the kernel, so we can't proceed.
+                    interactive_backend is not None and
+                    # There has to be an interactive backend (i.e. different
+                    # from inline) set in the kernel before. Else, a restart
+                    # is not necessary.
+                    interactive_backend != inline_backend and
+                    # The interactive backend to switch to has to be different
+                    # from the current one
+                    interactive_backend != pylab_backend_o
+                ):
+                    clients_backend_require_restart.append(True)
+
+                    # Detect if current client requires restart
+                    if id(client) == id(current_client):
+                        current_client_backend_require_restart = True
+
+                        # For testing
+                        if self._testing:
+                            os.environ['BACKEND_REQUIRE_RESTART'] = 'true'
+                else:
+                    clients_backend_require_restart.append(False)
+
+            pylab_restart = any(clients_backend_require_restart)
 
         if (restart_needed or pylab_restart) and not running_under_pytest():
             self.initial_conf_options = self.get_conf_options()
@@ -837,8 +887,11 @@ class IPythonConsoleWidget(PluginMainWidget):
         # Apply settings
         options = {option: value}
         for idx, client in enumerate(self.clients):
-            restart = ((pylab_restart and client_backend_not_inline[idx]) or
-                       restart_needed)
+            restart = (
+                (pylab_restart and clients_backend_require_restart[idx]) or
+                restart_needed
+            )
+
             if not (restart and restart_all) or no_restart:
                 sw = client.shellwidget
                 if sw.is_debugging() and sw._executing:
@@ -857,7 +910,7 @@ class IPythonConsoleWidget(PluginMainWidget):
                 client.restart_kernel()
                 client.ask_before_restart = current_ask_before_restart
 
-        if (((pylab_restart and current_client_backend_not_inline)
+        if (((pylab_restart and current_client_backend_require_restart)
              or restart_needed) and restart_current and current_client):
             current_client_ask_before_restart = (
                 current_client.ask_before_restart)
@@ -1003,7 +1056,10 @@ class IPythonConsoleWidget(PluginMainWidget):
         self.sig_shellwidget_created.emit(client.shellwidget)
 
     def _shellwidget_deleted(self, client):
-        self.sig_shellwidget_deleted.emit(client.shellwidget)
+        try:
+            self.sig_shellwidget_deleted.emit(client.shellwidget)
+        except RuntimeError:
+            pass
 
     def _create_client_for_kernel(self, connection_file, hostname, sshkey,
                                   password):
@@ -1033,6 +1089,9 @@ class IPythonConsoleWidget(PluginMainWidget):
         known_spyder_kernel = False
         slave_ord = ord('A') - 1
         kernel_manager = None
+        stderr_obj = None
+        stdout_obj = None
+        fault_obj = None
 
         for cl in self.clients:
             if connection_file in cl.connection_file:
@@ -1043,6 +1102,12 @@ class IPythonConsoleWidget(PluginMainWidget):
                     master_id = cl.id_['int_id']
                     is_external_kernel = cl.shellwidget.is_external_kernel
                     known_spyder_kernel = cl.shellwidget.is_spyder_kernel
+                    if cl.stderr_obj:
+                        stderr_obj = cl.stderr_obj.copy()
+                    if cl.stdout_obj:
+                        stdout_obj = cl.stdout_obj.copy()
+                    if cl.fault_obj:
+                        fault_obj = cl.fault_obj.copy()
                 given_name = cl.given_name
                 new_slave_ord = ord(cl.id_['str_id'])
                 if new_slave_ord > slave_ord:
@@ -1081,7 +1146,9 @@ class IPythonConsoleWidget(PluginMainWidget):
                               css_path=self.css_path,
                               configuration=self.CONFIGURATION,
                               handlers=self.registered_spyder_kernel_handlers,
-                              std_dir=self._test_dir)
+                              stderr_obj=stderr_obj,
+                              stdout_obj=stdout_obj,
+                              fault_obj=fault_obj)
 
         # Create kernel client
         kernel_client = QtKernelClient(connection_file=connection_file)
@@ -1307,6 +1374,8 @@ class IPythonConsoleWidget(PluginMainWidget):
     @Slot(object, object)
     def edit_file(self, filename, line):
         """Handle %edit magic petitions."""
+        if not osp.isfile(filename):
+            self.sig_edit_new.emit(filename)
         if encoding.is_text_file(filename):
             # The default line number sent by ipykernel is always the last
             # one, but we prefer to use the first.
@@ -1463,7 +1532,15 @@ class IPythonConsoleWidget(PluginMainWidget):
         self.master_clients += 1
         client_id = dict(int_id=str(self.master_clients),
                          str_id='A')
-        cf = self._new_connection_file()
+        std_dir = self._test_dir if self._test_dir else None
+        cf, km, kc, stderr_obj, stdout_obj = self.get_new_kernel(
+            is_cython, is_pylab, is_sympy, std_dir=std_dir)
+
+        if cf is not None:
+            fault_obj = StdFile(cf, '.fault', std_dir)
+        else:
+            fault_obj = None
+
         show_elapsed_time = self.get_conf('show_elapsed_time')
         reset_warning = self.get_conf('show_reset_namespace_warning')
         ask_before_restart = self.get_conf('ask_before_restart')
@@ -1487,7 +1564,9 @@ class IPythonConsoleWidget(PluginMainWidget):
                               css_path=self.css_path,
                               configuration=self.CONFIGURATION,
                               handlers=self.registered_spyder_kernel_handlers,
-                              std_dir=self._test_dir)
+                              stderr_obj=stderr_obj,
+                              stdout_obj=stdout_obj,
+                              fault_obj=fault_obj)
 
         self.add_tab(
             client, name=client.get_name(), filename=filename,
@@ -1505,24 +1584,36 @@ class IPythonConsoleWidget(PluginMainWidget):
             has_spyder_kernels = programs.is_module_installed(
                 'spyder_kernels',
                 interpreter=pyexec,
-                version='>=2.2.0;<2.3.0')
+                version=SPYDER_KERNELS_VERSION)
             if not has_spyder_kernels and not running_under_pytest():
                 client.show_kernel_error(
-                    _("Your Python environment or installation doesn't have "
-                      "the <tt>spyder-kernels</tt> module or the right "
-                      "version of it installed (>= 2.2.0 and < 2.3.0). "
+                    _("The Python environment or installation whose "
+                      "interpreter is located at"
+                      "<pre>"
+                      "    <tt>{0}</tt>"
+                      "</pre>"
+                      "doesn't have the <tt>spyder-kernels</tt> module or the "
+                      "right version of it installed ({1}). "
                       "Without this module is not possible for Spyder to "
                       "create a console for you.<br><br>"
-                      "You can install it by running in a system terminal:"
-                      "<br><br>"
-                      "<tt>conda install spyder-kernels=2.1</tt>"
-                      "<br><br>or<br><br>"
-                      "<tt>pip install spyder-kernels==2.1.*</tt>")
+                      "You can install it by activating your environment (if "
+                      "necessary) and then running in a system terminal:"
+                      "<pre>"
+                      "    <tt>{2}</tt>"
+                      "</pre>"
+                      "or"
+                      "<pre>"
+                      "    <tt>{3}</tt>"
+                      "</pre>").format(
+                          pyexec,
+                          SPYDER_KERNELS_VERSION_MSG,
+                          SPYDER_KERNELS_CONDA,
+                          SPYDER_KERNELS_PIP
+                      )
                 )
                 return
 
-        self.connect_client_to_kernel(client, is_cython=is_cython,
-                                      is_pylab=is_pylab, is_sympy=is_sympy)
+        self.connect_client_to_kernel(client, km, kc)
         if client.shellwidget.kernel_manager is None:
             return
         self.register_client(client, give_focus=give_focus)
@@ -1550,23 +1641,95 @@ class IPythonConsoleWidget(PluginMainWidget):
             self._create_client_for_kernel(connection_file, hostname, sshkey,
                                            password)
 
-    def connect_client_to_kernel(self, client, is_cython=False,
-                                 is_pylab=False, is_sympy=False):
-        """Connect a client to its kernel."""
-        connection_file = client.connection_file
-        stderr_handle = (
-            None if self._test_no_stderr else client.stderr_obj.handle)
-        stdout_handle = (
-            None if self._test_no_stderr else client.stdout_obj.handle)
+    def get_new_kernel(self, is_cython=False, is_pylab=False,
+                       is_sympy=False, std_dir=None):
+        """Get a new kernel, and cache one for next time."""
+        # Cache another kernel for next time.
+        kernel_spec = self.create_kernel_spec(
+            is_cython=is_cython,
+            is_pylab=is_pylab,
+            is_sympy=is_sympy
+        )
+
+        new_kernel = self.create_new_kernel(kernel_spec, std_dir)
+        if new_kernel[2] is None:
+            # error
+            self.close_cached_kernel()
+            return new_kernel
+
+        # Check cached kernel has the same configuration as is being asked
+        cached_kernel = None
+        if self._cached_kernel_properties is not None:
+            (cached_spec,
+             cached_env,
+             cached_argv,
+             cached_dir,
+             cached_kernel) = self._cached_kernel_properties
+            # Call interrupt_mode so the dict will be the same
+            kernel_spec.interrupt_mode
+            cached_spec.interrupt_mode
+            valid = (std_dir == cached_dir
+                     and cached_spec.__dict__ == kernel_spec.__dict__
+                     and kernel_spec.argv == cached_argv
+                     and kernel_spec.env == cached_env)
+            if not valid:
+                # Close the kernel
+                self.close_cached_kernel()
+                cached_kernel = None
+
+        # Cache the new kernel
+        self._cached_kernel_properties = (
+            kernel_spec,
+            kernel_spec.env,
+            kernel_spec.argv,
+            std_dir,
+            new_kernel)
+
+        if cached_kernel is None:
+            return self.create_new_kernel(kernel_spec, std_dir)
+
+        return cached_kernel
+
+    def close_cached_kernel(self):
+        """Close the cached kernel."""
+        if self._cached_kernel_properties is None:
+            return
+        cached_kernel = self._cached_kernel_properties[-1]
+        _, kernel_manager, _, stderr_obj, stdout_obj = cached_kernel
+        kernel_manager.stop_restarter()
+        kernel_manager.shutdown_kernel(now=True)
+        self._cached_kernel_properties = None
+        if stderr_obj:
+            stderr_obj.remove()
+        if stdout_obj:
+            stdout_obj.remove()
+
+    def create_new_kernel(self, kernel_spec, std_dir=None):
+        """Create a new kernel."""
+        connection_file = self._new_connection_file()
+        if connection_file is None:
+            return None, None, None, None, None
+
+        stderr_obj = None
+        stderr_handle = None
+        stdout_obj = None
+        stdout_handle = None
+        if not self._test_no_stderr:
+            stderr_obj = StdFile(connection_file, '.stderr', std_dir)
+            stderr_handle = stderr_obj.handle
+            stdout_obj = StdFile(connection_file, '.stdout', std_dir)
+            stdout_handle = stdout_obj.handle
+
         km, kc = self.create_kernel_manager_and_kernel_client(
             connection_file,
             stderr_handle,
             stdout_handle,
-            is_cython=is_cython,
-            is_pylab=is_pylab,
-            is_sympy=is_sympy,
+            kernel_spec,
         )
+        return connection_file, km, kc, stderr_obj, stdout_obj
 
+    def connect_client_to_kernel(self, client, km, kc):
+        """Connect a client to its kernel."""
         # An error occurred if this is True
         if isinstance(km, str) and kc is None:
             client.shellwidget.kernel_manager = None
@@ -1781,6 +1944,8 @@ class IPythonConsoleWidget(PluginMainWidget):
             open_clients.remove(client)
         # Close all closing shellwidgets.
         ShellWidget.wait_all_shutdown()
+        # Close cached kernel
+        self.close_cached_kernel()
         return True
 
     def get_client_index_from_id(self, client_id):
@@ -1925,15 +2090,8 @@ class IPythonConsoleWidget(PluginMainWidget):
     def create_kernel_manager_and_kernel_client(self, connection_file,
                                                 stderr_handle,
                                                 stdout_handle,
-                                                is_cython=False,
-                                                is_pylab=False,
-                                                is_sympy=False):
+                                                kernel_spec):
         """Create kernel manager and client."""
-        # Kernel spec
-        kernel_spec = self.create_kernel_spec(is_cython=is_cython,
-                                              is_pylab=is_pylab,
-                                              is_sympy=is_sympy)
-
         # Kernel manager
         try:
             kernel_manager = SpyderKernelManager(
@@ -2070,7 +2228,7 @@ class IPythonConsoleWidget(PluginMainWidget):
 
     # ---- For cells
     def run_cell(self, code, cell_name, filename, run_cell_copy,
-                 function='runcell'):
+                 focus_to_editor, function='runcell'):
         """Run cell in current or dedicated client."""
 
         def norm(text):
@@ -2097,10 +2255,16 @@ class IPythonConsoleWidget(PluginMainWidget):
                 line = code.strip()
 
             try:
-                self.execute_code(line)
+                self.execute_code(line, set_focus=not focus_to_editor)
             except AttributeError:
                 pass
-            self.sig_switch_to_plugin_requested.emit()
+
+            # This is necessary to prevent raising the console if the editor
+            # and console are tabified next to each other and the 'Maintain
+            # focus in the editor' option is activated.
+            # Fixes spyder-ide/spyder#17028
+            if not focus_to_editor:
+                self.sig_switch_to_plugin_requested.emit()
         else:
             # XXX: not sure it can really happen
             QMessageBox.warning(
@@ -2112,9 +2276,11 @@ class IPythonConsoleWidget(PluginMainWidget):
                 QMessageBox.Ok
             )
 
-    def debug_cell(self, code, cell_name, filename, run_cell_copy):
+    def debug_cell(self, code, cell_name, filename, run_cell_copy,
+                   focus_to_editor):
         """Debug current cell."""
-        self.run_cell(code, cell_name, filename, run_cell_copy, 'debugcell')
+        self.run_cell(code, cell_name, filename, run_cell_copy,
+                      focus_to_editor, 'debugcell')
 
     # ---- For scripts
     def run_script(self, filename, wdir, args, debug, post_mortem,
@@ -2167,15 +2333,17 @@ class IPythonConsoleWidget(PluginMainWidget):
                     # Fixes spyder-ide/spyder#7293.
                     pass
                 elif current_client:
-                    self.execute_code(line, current_client, clear_variables)
+                    self.execute_code(line, current_client, clear_variables,
+                                      set_focus=False)
                 else:
                     if is_new_client:
                         client.shellwidget.silent_execute('%clear')
                     else:
                         client.shellwidget.execute('%clear')
                     client.shellwidget.sig_prompt_ready.connect(
-                            lambda: self.execute_code(line, current_client,
-                                                      clear_variables))
+                            lambda: self.execute_code(
+                                line, current_client, clear_variables,
+                                set_focus=False))
             except AttributeError:
                 pass
             self.sig_switch_to_plugin_requested.emit()
@@ -2235,7 +2403,8 @@ class IPythonConsoleWidget(PluginMainWidget):
         self.active_project_path = active_project_path
 
     # ---- For execution
-    def execute_code(self, lines, current_client=True, clear_variables=False):
+    def execute_code(self, lines, current_client=True, clear_variables=False,
+                     set_focus=True):
         """Execute code instructions."""
         sw = self.get_current_shellwidget()
         if sw is not None:
@@ -2257,7 +2426,8 @@ class IPythonConsoleWidget(PluginMainWidget):
             except AttributeError:
                 pass
             self.activateWindow()
-            self.get_current_client().get_control().setFocus()
+            if set_focus:
+                self.get_current_client().get_control().setFocus()
 
     # ---- For error handling
     def go_to_error(self, text):
