@@ -9,20 +9,18 @@ Plots Main Widget.
 """
 
 # Third party imports
-from qtpy.QtCore import QPoint, Qt, Signal
-from qtpy.QtWidgets import QHBoxLayout, QSpinBox, QStackedWidget
+from qtpy.QtCore import Qt, Signal
+from qtpy.QtWidgets import QSpinBox
 
 # Local imports
 from spyder.api.config.decorators import on_conf_change
 from spyder.api.translations import get_translation
-from spyder.api.widgets.main_widget import (PluginMainWidgetMenus,
-                                            PluginMainWidget)
-from spyder.api.widgets.mixins import SpyderWidgetMixin
+from spyder.api.widgets.main_widget import PluginMainWidgetMenus
+from spyder.api.shellconnect.main_widget import ShellConnectMainWidget
 from spyder.plugins.plots.widgets.figurebrowser import FigureBrowser
 from spyder.utils.misc import getcwd_or_home
 from spyder.utils.palette import QStylePalette
 
-
 # Localization
 _ = get_translation('spyder')
 
@@ -56,75 +54,13 @@ class PlotsWidgetMainToolbarSections:
     Zoom = 'zoom_section'
 
 
+class PlotsWidgetToolbarItems:
+    ZoomSpinBox = 'zoom_spin'
+
+
 # --- Widgets
 # ----------------------------------------------------------------------------
-class PlotsStackedWidget(QStackedWidget, SpyderWidgetMixin):
-    # Signals
-    sig_thumbnail_menu_requested = Signal(QPoint, object)
-    """
-    This signal is emitted to request a context menu on the figure thumbnails.
-
-    Parameters
-    ----------
-    point: QPoint
-        The QPoint in global coordinates where the menu was requested.
-    figure_thumbnail: spyder.plugins.plots.widget.figurebrowser.FigureThumbnail
-        The clicked figure thumbnail.
-    """
-
-    sig_figure_menu_requested = Signal(QPoint)
-    """
-    This signal is emitted to request a context menu on the main figure
-    canvas.
-
-    Parameters
-    ----------
-    point: QPoint
-        The QPoint in global coordinates where the menu was requested.
-    """
-
-    sig_figure_loaded = Signal()
-    """This signal is emitted when a new figure is loaded."""
-
-    sig_save_dir_changed = Signal(str)
-    """
-    This signal is emitted to inform that the current folder where images are
-    saved has changed.
-
-    Parameters
-    ----------
-    save_dir: str
-        The new path where images are saved.
-    """
-
-    sig_zoom_changed = Signal(int)
-    """
-    This signal is emitted when zoom has changed.
-
-    Parameters
-    ----------
-    zoom_value: int
-        The new value for the zoom property.
-    """
-
-    def __init__(self, parent):
-        super().__init__(parent=parent, class_parent=parent)
-
-    def addWidget(self, widget):
-        """Override Qt method."""
-        if isinstance(widget, FigureBrowser):
-            widget.sig_figure_menu_requested.connect(
-                self.sig_figure_menu_requested)
-            widget.sig_thumbnail_menu_requested.connect(
-                self.sig_thumbnail_menu_requested)
-            widget.sig_figure_loaded.connect(self.sig_figure_loaded)
-            widget.sig_save_dir_changed.connect(self.sig_save_dir_changed)
-            widget.sig_zoom_changed.connect(self.sig_zoom_changed)
-
-        super().addWidget(widget)
-
-
-class PlotsWidget(PluginMainWidget):
+class PlotsWidget(ShellConnectMainWidget):
     sig_figure_loaded = Signal()
     """This signal is emitted when a figure is loaded succesfully"""
 
@@ -143,9 +79,8 @@ class PlotsWidget(PluginMainWidget):
         super().__init__(name, plugin, parent)
 
         # Widgets
-        self._stack = PlotsStackedWidget(parent=self)
-        self._shellwidgets = {}
         self.zoom_disp = QSpinBox(self)
+        self.zoom_disp.ID = PlotsWidgetToolbarItems.ZoomSpinBox
         self._right_clicked_thumbnail = None
 
         # Widget setup
@@ -156,24 +91,11 @@ class PlotsWidget(PluginMainWidget):
         self.zoom_disp.setRange(0, 9999)
         self.zoom_disp.setValue(100)
 
-        # Layout
-        layout = QHBoxLayout()
-        layout.setSpacing(0)
-        layout.setContentsMargins(0, 0, 0, 0)
-        layout.addWidget(self._stack)
-        self.setLayout(layout)
+        # Resize to a huge width to get the right size of the thumbnail
+        # scrollbar at startup.
+        self.resize(50000, self.height())
 
-        # Signals
-        self._stack.sig_figure_loaded.connect(self.sig_figure_loaded)
-        self._stack.sig_figure_menu_requested.connect(self.show_figure_menu)
-        self._stack.sig_thumbnail_menu_requested.connect(
-            self.show_thumbnail_menu)
-        self._stack.sig_zoom_changed.connect(self.zoom_disp.setValue)
-        self._stack.sig_figure_loaded.connect(self.update_actions)
-        self._stack.sig_save_dir_changed.connect(
-            lambda val: self.set_conf('save_dir', val))
-
-    # --- PluginMainWidget API
+    # ---- PluginMainWidget API
     # ------------------------------------------------------------------------
     def get_title(self):
         return _('Plots')
@@ -318,27 +240,31 @@ class PlotsWidget(PluginMainWidget):
             value = figviewer.figcanvas.fig is not None
 
         for __, action in self.get_actions().items():
-            if action and action not in [self.mute_action,
-                                         self.outline_action,
-                                         self.fit_action,
-                                         self.undock_action,
-                                         self.close_action,
-                                         self.dock_action]:
-                action.setEnabled(value)
+            try:
+                if action and action not in [self.mute_action,
+                                             self.outline_action,
+                                             self.fit_action,
+                                             self.undock_action,
+                                             self.close_action,
+                                             self.dock_action,
+                                             self.toggle_view_action]:
+                    action.setEnabled(value)
 
-                # IMPORTANT: Since we are defining the main actions in here
-                # and the context is WidgetWithChildrenShortcut we need to
-                # assign the same actions to the children widgets in order
-                # for shortcuts to work
-                if figviewer:
-                    figviewer_actions = figviewer.actions()
-                    thumbnails_sb_actions = thumbnails_sb.actions()
+                    # IMPORTANT: Since we are defining the main actions in here
+                    # and the context is WidgetWithChildrenShortcut we need to
+                    # assign the same actions to the children widgets in order
+                    # for shortcuts to work
+                    if figviewer:
+                        figviewer_actions = figviewer.actions()
+                        thumbnails_sb_actions = thumbnails_sb.actions()
 
-                    if action not in figviewer_actions:
-                        figviewer.addAction(action)
+                        if action not in figviewer_actions:
+                            figviewer.addAction(action)
 
-                    if action not in thumbnails_sb_actions:
-                        thumbnails_sb.addAction(action)
+                        if action not in thumbnails_sb_actions:
+                            thumbnails_sb.addAction(action)
+            except RuntimeError:
+                pass
 
         self.zoom_disp.setEnabled(value)
 
@@ -358,123 +284,38 @@ class PlotsWidget(PluginMainWidget):
                 widget.setup({option: value})
                 self.update_actions()
 
-    # --- Public API:
+    # ---- Public API:
     # ------------------------------------------------------------------------
-    def set_current_widget(self, fig_browser):
-        """
-        Set the current figure browser widget in the stack.
 
-        Parameters
-        ----------
-        fig_browser: spyder.plugins.plots.widgets.figurebrowser.FigureBrowser
-            The widget to set.
-        """
-        self._stack.setCurrentWidget(fig_browser)
+    def create_new_widget(self, shellwidget):
+        fig_browser = FigureBrowser(parent=self,
+                                    background_color=MAIN_BG_COLOR)
+        fig_browser.update_splitter_widths(self.width())
+        fig_browser.set_shellwidget(shellwidget)
+        fig_browser.sig_redirect_stdio_requested.connect(
+            self.sig_redirect_stdio_requested)
 
-    def current_widget(self):
-        """
-        Return the current figure browser widget in the stack.
+        fig_browser.sig_figure_menu_requested.connect(
+            self.show_figure_menu)
+        fig_browser.sig_thumbnail_menu_requested.connect(
+            self.show_thumbnail_menu)
+        fig_browser.sig_figure_loaded.connect(self.update_actions)
+        fig_browser.sig_save_dir_changed.connect(
+            lambda val: self.set_conf('save_dir', val))
+        fig_browser.sig_zoom_changed.connect(self.zoom_disp.setValue)
+        return fig_browser
 
-        Returns
-        -------
-        spyder.plugins.plots.widgets.figurebrowser.FigureBrowser
-            The current widget.
-        """
-        return self._stack.currentWidget()
+    def close_widget(self, fig_browser):
+        fig_browser.close()
 
-    def count(self):
-        """
-        Return the number of widgets in the stack.
-
-        Returns
-        -------
-        int
-            The number of widgets in the stack.
-        """
-        return self._stack.count()
-
-    def remove_widget(self, fig_browser):
-        """
-        Remove widget from stack.
-
-        Parameters
-        ----------
-        fig_browser: spyder.plugins.plots.widgets.figurebrowser.FigureBrowser
-            The figure browser widget to remove.
-        """
-        self._stack.removeWidget(fig_browser)
-
-    def add_widget(self, fig_browser):
-        """
-        Add widget to stack.
-
-        Parameters
-        ----------
-        fig_browser: spyder.plugins.plots.widgets.figurebrowser.FigureBrowser
-            The figure browser widget to add.
-        """
-        self._stack.addWidget(fig_browser)
-
-    def add_shellwidget(self, shellwidget):
-        """
-        Add a new shellwidget registered with the plots plugin.
-
-        This function registers a new FigureBrowser for browsing the figures
-        in the shell.
-
-        Parameters
-        ----------
-        shelwidget: spyder.plugins.ipyconsole.widgets.shell.ShellWidget
-            The shell widget.
-        """
-        shellwidget_id = id(shellwidget)
-        if shellwidget_id not in self._shellwidgets:
-            fig_browser = FigureBrowser(parent=self._stack,
-                                        background_color=MAIN_BG_COLOR)
-            fig_browser.update_splitter_widths(self.width())
-            fig_browser.set_shellwidget(shellwidget)
-            fig_browser.sig_redirect_stdio_requested.connect(
-                self.sig_redirect_stdio_requested)
-            self.add_widget(fig_browser)
-            self._shellwidgets[shellwidget_id] = fig_browser
-            self.set_shellwidget(shellwidget)
-            return fig_browser
-
-    def remove_shellwidget(self, shellwidget):
-        """
-        Remove the shellwidget registered with the plots plugin.
-
-        Parameters
-        ----------
-        shelwidget: spyder.plugins.ipyconsole.widgets.shell.ShellWidget
-            The shell widget.
-        """
-        shellwidget_id = id(shellwidget)
-        if shellwidget_id in self._shellwidgets:
-            fig_browser = self._shellwidgets.pop(shellwidget_id)
-            self.remove_widget(fig_browser)
-            fig_browser.close()
-
-    def set_shellwidget(self, shellwidget):
-        """
-        Update the current shellwidget displayed with the plots plugin.
-
-        Parameters
-        ----------
-        shelwidget: spyder.plugins.ipyconsole.widgets.shell.ShellWidget
-            The shell widget.
-        """
+    def switch_widget(self, fig_browser, old_fig_browser):
         option_keys = [('auto_fit_plotting', True),
                        ('mute_inline_plotting', True),
                        ('show_plot_outline', True),
                        ('save_dir', getcwd_or_home())]
 
         conf_values = {k: self.get_conf(k, d) for k, d in option_keys}
-        shellwidget_id = id(shellwidget)
-        if shellwidget_id in self._shellwidgets:
-            fig_browser = self._shellwidgets[shellwidget_id]
-            fig_browser.setup(conf_values)
-            self.set_current_widget(fig_browser)
+        fig_browser.setup(conf_values)
 
     def show_figure_menu(self, qpoint):
         """
