@@ -241,19 +241,25 @@ class DataFrameModel(QAbstractTableModel):
         if self.df.shape[0] == 0: # If no rows to compute max/min then return
             return
         self.max_min_col = []
-        for dummy, col in self.df.iteritems():
-            if col.dtype in REAL_NUMBER_TYPES + COMPLEX_NUMBER_TYPES:
-                if col.dtype in REAL_NUMBER_TYPES:
-                    vmax = col.max(skipna=True)
-                    vmin = col.min(skipna=True)
+        for __, col in self.df.iteritems():
+            # This is necessary to catch an error in Pandas when computing
+            # the maximum of a column.
+            # Fixes spyder-ide/spyder#17145
+            try:
+                if col.dtype in REAL_NUMBER_TYPES + COMPLEX_NUMBER_TYPES:
+                    if col.dtype in REAL_NUMBER_TYPES:
+                        vmax = col.max(skipna=True)
+                        vmin = col.min(skipna=True)
+                    else:
+                        vmax = col.abs().max(skipna=True)
+                        vmin = col.abs().min(skipna=True)
+                    if vmax != vmin:
+                        max_min = [vmax, vmin]
+                    else:
+                        max_min = [vmax, vmin - 1]
                 else:
-                    vmax = col.abs().max(skipna=True)
-                    vmin = col.abs().min(skipna=True)
-                if vmax != vmin:
-                    max_min = [vmax, vmin]
-                else:
-                    max_min = [vmax, vmin - 1]
-            else:
+                    max_min = None
+            except TypeError:
                 max_min = None
             self.max_min_col.append(max_min)
 
@@ -529,8 +535,8 @@ class DataFrameView(QTableView, SpyderConfigurationAccessor):
         self.setModel(model)
         self.setHorizontalScrollBar(hscroll)
         self.setVerticalScrollBar(vscroll)
-        self.setHorizontalScrollMode(1)
-        self.setVerticalScrollMode(1)
+        self.setHorizontalScrollMode(QTableView.ScrollPerPixel)
+        self.setVerticalScrollMode(QTableView.ScrollPerPixel)
 
         self.sort_old = [None]
         self.header_class = header
@@ -751,7 +757,7 @@ class DataFrameHeaderModel(QAbstractTableModel):
             if orientation == Qt.Horizontal:
                 return Qt.AlignCenter
             else:
-                return Qt.AlignRight | Qt.AlignVCenter
+                return int(Qt.AlignRight | Qt.AlignVCenter)
         if role != Qt.DisplayRole and role != Qt.ToolTipRole:
             return None
         if self.axis == 1 and self._shape[1] <= 1:
@@ -843,7 +849,7 @@ class DataFrameLevelModel(QAbstractTableModel):
             if orientation == Qt.Horizontal:
                 return Qt.AlignCenter
             else:
-                return Qt.AlignRight | Qt.AlignVCenter
+                return int(Qt.AlignRight | Qt.AlignVCenter)
         if role != Qt.DisplayRole and role != Qt.ToolTipRole:
             return None
         if self.model.header_shape[0] <= 1 and orientation == Qt.Horizontal:
